/*
 * TABLE.H
 * Header file for table.c users
 * By Joshua Levy  
 */

#ifndef TABLE_H
#define TABLE_H 1

typedef struct {     /* not currently used, should be a range of numbers */
    int first ;
    int last ;
} range_t ;

#include "tablesimple.h"
#include "tablemulti.h"
#include "tablesimpbig.h"

typedef struct {
    int type ;
    char *title ;
    char roll[10] ;
    char *source ;
    int num ;
} alltable_t ;

typedef union {      /* This is the handle users use to manipulate tables. */
    int type ;
    alltable_t all ;
    simpletable_t simple ;
    multitable_t multi ;
} table_t ;

/*
 * GENERAL PURPOSE MACROS
 */
#define TB_WHITE  "\t \n\f"  /* white space */

/*
 * error return values
 */
#define TB_OK         0      /* success */
#define TB_E_MISC    -1      /* general error: something went wrong */
#define TB_E_NOTYET  -2      /* feature or subroutine not yet implemented */
#define TB_E_NOFILE  -3      /* file doesn't exist */
#define TB_E_NO_COL  -4      /* no column in the table with that name */
#define TB_E_NOTYPE  -5      /* that type of table is not supported */
#define TB_E_TOOMANY -6      /* there are not that many possible results */

/*
 * lint help
 */
#define FREE(ptr)      (void)free((char*)ptr)
#define MALLOC(size)   malloc((unsigned int)size)
#define FCLOSE(fp)     (void)fclose(fp)


/*
 * functional interface
 */
extern int tb_read() ;
extern int tb_print() ;
extern int tb_use() ;
extern int tb_free() ;
extern int tb_many() ;
extern int tb_query() ;
#ifdef NEEDDUP
extern char *strdup() ;
#endif

/*
 * Queries
 */
#include "random.h"
#define TB_Q_MIN RA_MIN
#define TB_Q_MAX RA_MAX
#define TB_Q_AVE RA_AVE
#define TB_Q_LEGAL RA_LEGAL


#endif
