/*
 * TABLEMULTI.C
 * Code supporting rolling on multiple collumn tables
 * By Joshua Levy  
 *
 * COMPILING
 *
% cc -g -c tablemulti.c 
 */

#include <stdio.h>
#include <assert.h>
#include <string.h>
#include <ctype.h>

#include "table.h"

int tb_multi_read(rt,fp,title) table_t **rt ; FILE *fp ; char *title ; 
{
    register multitable_t *t ;
    register int c,r ;
    char buf[1000] ;
    char roll[10] ;
    int rows,cols ;
    int sts ;

    sts = (int)fgets(buf,sizeof(buf),fp) ;
    assert(sts) ;
    sts = sscanf(buf,"%s %d %d",roll,&rows,&cols) ;
    assert(sts==3) ;
    t = (multitable_t*)
	MALLOC(sizeof(multitable_t)+sizeof(multirow_t*)*(rows-1)) ;
    assert(t) ;
    t->title = strdup(title) ;
    strncpy(t->roll,roll,sizeof(t->roll)) ;
    t->numrows = rows ;
    t->numcols = cols ;
    sts = (int)fgets(buf,sizeof(buf),fp) ;
    assert(sts) ;
    t->source = strdup(strtok(buf,"\n")) ;
    /* read in the column names */
    t->colnames = (char **)calloc(cols+1,sizeof(char*)) ;
    assert(t->colnames) ;
    sts = (int)fgets(buf,sizeof(buf),fp) ;
    assert(sts) ;
    assert(strtok(buf," \t\n")) ; /* get rid of first dash */
    for(c=0 ; c<t->numcols ; c++) {
	t->colnames[c]=strdup(strtok((char*)0,"\t\n")) ;
    }
    t->colnames[c] = NULL ;

    for(r=0 ; r<rows ; r++) {
	t->rows[r] = (multirow_t*)
	    MALLOC(sizeof(multirow_t)+sizeof(char*)*(t->numcols-1)) ;
	sts = (int)fgets(buf,sizeof(buf),fp) ;
	assert(sts) ;
	sts=parse_range(strtok(buf,"\t\n"),&t->rows[r]->first,
			&t->rows[r]->last);
	assert(sts>0) ;
	for(c=0 ; c<t->numcols ; c++) {
	    t->rows[r]->result[c] = strdup(strtok((char*)0,"\t\n")) ;
	    assert(t->rows[r]->result[c]) ;
	}

    }

    *rt = (table_t*)t ;
    return 0 ;
}

int tb_multi_use(tab,col,result) table_t *tab ; char *col ;char **result ;
{
    register multitable_t *t ;
    int ncol ;
    int i,r ;

    t = (multitable_t*)tab ;

    for(r=0 ; r<t->numcols ; r++) 
	if (!strcmp(t->colnames[r],col)) {
	    ncol = r ;
	    break ;
	}
    if (r==t->numcols)
	return TB_E_NO_COL ;

    r = ra_roll(t->roll) ;

    for(i=0 ; i<t->numrows ; i++) {
	if (r>=t->rows[i]->first && r<=t->rows[i]->last) {
	    *result = t->rows[i]->result[ncol] ;
	    return TB_OK ;
	}
    }

    return TB_E_MISC ;
}

int tb_multi_print(tab,fp,way) table_t *tab ; FILE *fp ; int way ;
{
    register multitable_t *t ;
    int r,c ;

    t = (multitable_t*)tab ;
    switch (way) {
      case 1:
	fprintf(fp,".TS\ncB\ncI\ncI.\n") ;
	fprintf(fp,"%s\n",t->title) ;
	fprintf(fp,"(roll %s)\n",t->roll) ;
	fprintf(fp,"%s\n",t->source) ;
	fprintf(fp,".T&\n") ;
	fprintf(fp,"rI ") ;
	for(c=0 ; c<t->numcols ; c++)
	    fprintf(fp,"c%c",c==t->numcols-1?'.':' ') ;
	fprintf(fp,"\nRoll\t") ;
	for(c=0 ; c<t->numcols ; c++)
	    fprintf(fp,"%s%c",t->colnames[c],
		    c==t->numcols-1?'\n':'\t') ;
	
	for(r=0 ; r<t->numrows ; r++) {
	    if (t->rows[r]->first==t->rows[r]->last)
		fprintf(fp,"%d\t",t->rows[r]->first) ;
	    else
		fprintf(fp,"%d-%d\t",t->rows[r]->first,t->rows[r]->last) ;
	    for(c=0 ; c<t->numcols ; c++)
		fprintf(fp,"%s%c",t->rows[r]->result[c],
			c==t->numcols-1?'\n':'\t') ;
	}
	fprintf(fp,".TE\n") ;
	break ;
      default:
	return TB_E_NOTYET ;
    }
    return TB_OK ;
}

int tb_multi_free(t) table_t *t ;
{
    int r,c ;
    multitable_t *mt ;

    mt = (multitable_t *)t ;

    FREE(mt->title) ;
    FREE(mt->source) ;
    for(c=0 ; c<mt->numcols ; c++) {
	FREE(mt->colnames[c]) ;
    }
    for(r=0 ; r<mt->numrows ; r++) {
	FREE(mt->rows[r]) ;
	for(c=0 ; c<mt->numcols ; c++) {
	    FREE(mt->rows[r]->result[c]) ;
	}
    }
    FREE(mt->colnames) ;
    FREE(mt) ;

    return TB_OK ;
}

int tb_multi_choose(t) table_t *t ;
{
    return TB_E_NOTYET ;
}

int tb_multi_back(t,find,data,res) table_t *t ; char *find, *data, **res ;
{
    multitable_t *tab = (multitable_t *)t ;
    int i,k ; 

    assert(t) ;
    assert(find) ;
    assert(res) ;

    *res = NULL ;
    if (!data) {
	/* If no data, search the whole table */
	for(i=0 ; i<tab->numrows ; i++) {
	    for(i=0 ; i<tab->numcols ; i++) {
		if (!strcmp(find,tab->rows[i]->result[k])) {
		    *res = tab->rows[i]->result[k] ;
		    break ;
		}
	    }
	}
    } else {
	/* other wise, just search the row in question */
	for(i=0 ; i<tab->numcols ; i++) {
	    if (!strcmp(data,tab->colnames[k]))
		break ;
	}
	if (k==tab->numcols)
	    return TB_E_MISC ;
	for(i=0 ; i<tab->numrows ; i++) {
	    if (!strcmp(find,tab->rows[i]->result[k])) {
		*res = (char*)&tab->rows[i]->first ;
		break ;
	    }
	}
    }
    return TB_OK ;
}
