/*
 * TABLESIMPLE.C
 * Code supporting rolling on simple tables
 * By Joshua Levy  
 *
 * COMPILING
 *
% cc -g -c tablesimple.c 
 */

#include <stdio.h>
#include <assert.h>
#include <string.h>
#include <ctype.h>

#include "table.h"

int parse_range(str,first,last) char *str ; int *first ; int *last ;
{
    int n ;

    n = sscanf(str,"%d-%d",first,last) ;
    if (n==1)
	*first = *last = atoi(str) ;
    return n ;
}


int tb_simple_read(rt,fp,title) table_t **rt ; FILE *fp ; char *title ; 
{
    simpletable_t *t ;
    char buf[1000] ;
    char roll[10] ;
    int rows ;
    int sts ;
    int r ;

    sts = (int)fgets(buf,sizeof(buf),fp) ;
    assert(sts) ;
    sts = sscanf(buf,"%s %d",roll,&rows) ;
    assert(sts==2) ;
    t = (simpletable_t*)
	MALLOC(sizeof(simpletable_t)+sizeof(simplerow_t)*(rows-1)) ;
    assert(t) ;
    t->title = strdup(title) ;
    strncpy(t->roll,roll,sizeof(t->roll)) ;
    t->numrows = rows ;
    sts = (int)fgets(buf,sizeof(buf),fp) ;
    assert(sts) ;
    t->source = strdup(strtok(buf,"\n")) ;
    for(r=0 ; r<rows ; r++) {
	sts = (int)fgets(buf,sizeof(buf),fp) ;
	assert(sts) ;
	sts=parse_range(strtok(buf,"\t\n"),&t->rows[r].first,&t->rows[r].last);
	assert(sts>0) ;
	t->rows[r].result = strdup(strtok((char*)0,"\n")) ;
	assert(t->rows[r].result) ;
    }
    *rt = (table_t*)t ;
    return 0 ;
}

int tb_simple_use(tab,result) table_t *tab ; char **result ;
{
    simpletable_t *t ;
    int r,i ;

    t = (simpletable_t*)tab ;
    r = ra_roll(t->roll) ;
    for(i=0 ; i<t->numrows ; i++) {
	if (r>=t->rows[i].first && r<=t->rows[i].last) {
	    *result = t->rows[i].result ;
	    return TB_OK ;
	}
    }
    *result = NULL ;
    return -1 ;
}

int tb_simple_print(tab,fp,way) table_t *tab ; FILE *fp ; int way ;
{
    simpletable_t *t ;
    char tmp[10] ;
    int r ;

    t = (simpletable_t*)tab ;
    switch (way) {
      case 1:
	fprintf(fp,".TS\ncB s\ncI s.\n") ;
	fprintf(fp,"%s\n",t->title) ;
	fprintf(fp,"(roll %s)\n",t->roll) ;
	fprintf(fp,".T&\n") ;
	fprintf(fp,"r l.\n") ;
	for(r=0 ; r<t->numrows ; r++) {
	    if (t->rows[r].first==t->rows[r].last)
		sprintf(tmp,"%d",t->rows[r].first) ;
	    else
		sprintf(tmp,"%d-%d",t->rows[r].first,t->rows[r].last) ;
	    fprintf(fp,"%s\t%s\n",tmp,t->rows[r].result) ;
	}
	fprintf(fp,".TE\n") ;
	break ;
      default:
	return TB_E_NOTYET ;
    }
    return TB_OK ;
}

int tb_simple_free(t) table_t *t ;
{
    int r ;
    simpletable_t *st ;

    st = (simpletable_t*)t ;
    FREE(st->title) ;
    FREE(st->source) ;
    for(r=0 ; r<st->numrows ; r++)
	FREE(st->rows[r].result) ;    
    FREE(st) ;
    return TB_OK ;
}

int tb_simple_choose(t) table_t *t ;
{
    return TB_E_NOTYET ;
}

int tb_simple_back(t,find,res) table_t *t ; char *find ; char **res ;
{
    simpletable_t *tab = (simpletable_t *)t ;
    int i ;

    assert(find) ;
    assert(res) ;
    assert(t) ;

    *res = NULL ;
    for(i=0 ; i<tab->numrows ; i++) {
	if (!strcmp(find,tab->rows[i].result)) {
	    *res = (char*)&tab->rows[i].first ;
	    break ;
	}
    }
    return TB_OK ;
}
