/*
 * TBTEST.C
 * Test program for the tb (table) subsystem.
 * By Joshua levy  
 */

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <assert.h>
#include <errno.h>

#include "random.h"
#include "table.h"
 
main(argc,argv) int argc ; char *argv[] ;
{
    table_t *table ;
    char line[200] ;
    char *result ;
    char *results[10] ;
    char *comm ;
    char *arg1, *arg2, *arg3 ;
    int count, i ;
    int sts=0 ;
    int n ;
    int quer, num ;

    while (fgets(line,sizeof(line),stdin)) {
	comm = strtok(line," \t\n") ;
	if (!comm || comm[0]=='#')
	    continue ;
	arg1 = strtok(NULL,TB_WHITE) ;
	arg2 = strtok(NULL,TB_WHITE) ;
	arg3 = strtok(NULL,TB_WHITE) ;
	if (!strcmp(comm,"quit")) {
	    exit(0) ;
	} else if (!strcmp(comm,"read")) {
	    sts = tb_read(arg1,&table) ;
	    if (sts!=TB_OK) 
		printf("tb_read returned an error (%d) errno=%d\n",sts,errno) ;
	} else if (!strcmp(comm,"free")) {
	    sts = tb_free(table) ;
	    if (sts!=TB_OK) 
		printf("tb_print returned an error (%d)\n",sts) ;
	} else if (!strcmp(comm,"use")) {
	    switch (table->type) {
	      case 1:
	      case 3:
		count = arg1?atoi(arg1):0 ;
		for(i=0 ; i<count ; i++) {
		    sts = tb_use(table,&results[i]) ;
		}
		break ;
	      case 2:
		count = arg2?atoi(arg2):0 ;
		for(i=0 ; i<count ; i++) {
		    sts = tb_use(table,arg1,&results[i]) ;
		}
		break ;
	      default:
		printf("Table is of an unknown type\n") ;
	    }
	    if (sts==TB_OK) {
		for(i=0 ; i<count ; i++) 
		    printf("%d %s\n",i+1,results[i]) ;
	    } else {
		printf("tb_use returned an error (%d)\n",sts) ;
	    }
	} else if (!strcmp(comm,"back")) {
	    switch (table->type) {
	      case 1:
	      case 3:
		sts = tb_back(table,arg1,&result) ;
		break ;
	      case 2:
		sts = tb_back(table,arg1,arg2,&result) ;
		break ;
	      default:
		printf("Table is of an unknown type\n") ;
	    }
	    if (sts==TB_OK) {
		printf("%d\n",result?*(int*)result:-1) ;
	    } else {
		printf("tb_back returned an error (%d)\n",sts) ;
	    }
	} else if (!strcmp(comm,"many")) {
	    switch (table->type) {
	      case 1:
	      case 3:
		n = atoi(arg1) ;
		sts = tb_many(table,n,results) ;
		break ;
	      case 2:
		n = atoi(arg2) ;
		sts = tb_many(table,n,arg1,results) ;
		break ;
	      default:
		printf("Table is of an unknown type\n") ;
	    }
	    if (sts==TB_OK) {
		for(i=0 ; i<n ; i++) 
		    printf("%d %s\n",i,results[i]) ;
	    } else {
		printf("tb_many returned an error (%d)\n",sts) ;
	    }
	} else if (!strcmp(comm,"print")) {
	    sts = tb_print(table,"",1) ;
	    if (sts!=TB_OK) 
		printf("tb_print returned an error (%d)\n",sts) ;
	} else if (!strcmp(comm,"query")) {
	    if (!strcmp(arg1,"max")) quer = TB_Q_MAX ;
	    if (!strcmp(arg1,"min")) quer = TB_Q_MIN ;
	    if (!strcmp(arg1,"ave")) quer = TB_Q_AVE ;
	    if (!strcmp(arg1,"legal")) quer = TB_Q_LEGAL ;
	    sts = tb_query(table,quer,&num) ;
	    if (sts==TB_OK)
		printf("%d\n",num) ;
            else
		printf("tb_query returned an error (%d)\n",sts) ;
	} else if (!strcmp(comm,"help")) {
	    printf("Here is a BNF of the commands:\n") ;
	    printf("\tquit\n") ;
	    printf("\tread <file>\n") ;
	    printf("\tfree\n") ;
	    printf("\tuse [<column>] <count>\n") ;
	    printf("\tback\n") ;
	    printf("\tmany [<column>] <count>\n") ;
	    printf("\tprint\n") ;
	    printf("\tquery <query>\n") ;
	    printf("\thelp\n") ;
	} else {
	    printf("Illegal command.\n") ;
	}
    }
}


